/*
 * Copyright (c) Contributors to the Open 3D Engine Project.
 * For complete copyright and license terms please see the LICENSE at the root of this distribution.
 *
 * SPDX-License-Identifier: Apache-2.0 OR MIT
 *
 */

/*
    NOTE: To minimize your compile times, please consider manually including only the required header files instead of using this main include.
*/

#pragma once

#include "Actor.h"
#include "ActorInstance.h"
#include "ActorManager.h"
#include "ActorUpdateScheduler.h"
#include "Algorithms.h"
#include "Attachment.h"
#include "AttachmentSkin.h"
#include "AttachmentNode.h"
#include "BaseObject.h"
#include "AnimGraph.h"
#include "AnimGraphAttributeTypes.h"
#include "AnimGraphBindPoseNode.h"
#include "AnimGraphEntryNode.h"
#include "AnimGraphEventBuffer.h"
#include "AnimGraphExitNode.h"
#include "AnimGraphGameControllerSettings.h"
#include "AnimGraphInstance.h"
#include "AnimGraphManager.h"
#include "AnimGraphMotionCondition.h"
#include "AnimGraphMotionNode.h"
#include "AnimGraphNode.h"
#include "AnimGraphNodeData.h"
#include "AnimGraphNodeGroup.h"
#include "AnimGraphObject.h"
#include "AnimGraphObjectData.h"
#include "AnimGraphObjectFactory.h"
#include "AnimGraphParameterCondition.h"
#include "AnimGraphPlayTimeCondition.h"
#include "AnimGraphPose.h"
#include "AnimGraphPosePool.h"
#include "AnimGraphRefCountedData.h"
#include "AnimGraphRefCountedDataPool.h"
#include "AnimGraphStateCondition.h"
#include "AnimGraphStateMachine.h"
#include "AnimGraphStateTransition.h"
#include "AnimGraphSyncTrack.h"
#include "AnimGraphTimeCondition.h"
#include "AnimGraphTransitionCondition.h"
#include "AnimGraphVector2Condition.h"
#include "BlendSpace2DNode.h"
#include "BlendSpace1DNode.h"
#include "BlendTree.h"
#include "BlendTreeAccumTransformNode.h"
#include "BlendTreeBlend2Node.h"
#include "BlendTreeBlendNNode.h"
#include "BlendTreeBoolLogicNode.h"
#include "BlendTreeConnection.h"
#include "BlendTreeDirectionToWeightNode.h"
#include "BlendTreeFinalNode.h"
#include "BlendTreeFloatConditionNode.h"
#include "BlendTreeFloatMath1Node.h"
#include "BlendTreeFloatMath2Node.h"
#include "BlendTreeFloatSwitchNode.h"
#include "BlendTreeLookAtNode.h"
#include "BlendTreeMaskNode.h"
#include "BlendTreeMirrorPoseNode.h"
#include "BlendTreeMotionFrameNode.h"
#include "BlendTreeParameterNode.h"
#include "BlendTreePoseSwitchNode.h"
#include "BlendTreeRangeRemapperNode.h"
#include "BlendTreeSmoothingNode.h"
#include "BlendTreeTransformNode.h"
#include "BlendTreeTwoLinkIKNode.h"
#include "BlendTreeVector2ComposeNode.h"
#include "BlendTreeVector2DecomposeNode.h"
#include "BlendTreeVector3ComposeNode.h"
#include "BlendTreeVector3DecomposeNode.h"
#include "BlendTreeVector3Math1Node.h"
#include "BlendTreeVector3Math2Node.h"
#include "BlendTreeVector4ComposeNode.h"
#include "BlendTreeVector4DecomposeNode.h"
#include "CompressedKeyFrames.h"
#include "DualQuatSkinDeformer.h"
#include "EMotionFX.h"
#include "EMotionFXConfig.h"
#include "EMotionFXManager.h"
#include "EventHandler.h"
#include "EventInfo.h"
#include "EventManager.h"
#include "KeyFrame.h"
#include "KeyFrameFinder.h"
#include "KeyTrackLinearDynamic.h"
#include "LayerPass.h"
#include "Material.h"
#include "MemoryCategories.h"
#include "Mesh.h"
#include "MeshDeformer.h"
#include "MeshDeformerStack.h"
#include "MorphMeshDeformer.h"
#include "MorphSetup.h"
#include "MorphSetupInstance.h"
#include "MorphTarget.h"
#include "MorphTargetStandard.h"
#include "Motion.h"
#include "MotionEvent.h"
#include "MotionEventTable.h"
#include "MotionEventTrack.h"
#include "MotionInstance.h"
#include "MotionInstancePool.h"
#include "MotionLayerSystem.h"
#include "MotionManager.h"
#include "MotionQueue.h"
#include "MotionSet.h"
#include "MotionSystem.h"
#include "MultiThreadScheduler.h"
#include "Node.h"
#include "NodeAttribute.h"
#include "NodeGroup.h"
#include "NodeMap.h"
#include "PlayBackInfo.h"
#include "Pose.h"
#include "Recorder.h"
#include "RepositioningLayerPass.h"
#include "SingleThreadScheduler.h"
#include "Skeleton.h"
#include "SkinningInfoVertexAttributeLayer.h"
#include "SoftSkinDeformer.h"
#include "SoftSkinManager.h"
#include "StandardMaterial.h"
#include "SubMesh.h"
#include "ThreadData.h"
#include "Transform.h"
#include "TransformData.h"
#include "VertexAttributeLayer.h"
#include "VertexAttributeLayerAbstractData.h"
#include "Importer/ChunkProcessors.h"
#include "Importer/Importer.h"
